{ Globals for SINGLE LASER PHOTOPLOTTER program }
unit photogbl;

interface
         uses WinTypes;
{$I photodef.pas}

const
  file_name_len = 40;
  pdma_base = $0300;             { base I/O address of card }
  max_aperture = 99;             { max d_code number }
  dots_per_inch = 1000;
  max_aperture_size = 510;      { hard limit on aperture size }
  max_ap_area = 65520;          { max size of aperture bitmap }
  max_line = 15000;             { hard limit on raster line size }
  max_band_length = 150;	  { hard limit on size of bands }
  memory = 2400000;       { amount of heap available for band buffers }
  int_level = 7;                { interrupt level for PDMA card }
  int_num = $0f;                { number of interrupt }

{ define bits in control register }
  cr_mot_start = 1;
  cr_step_forw = 2;
  cr_1_step    = 4;
  cr_ext_step  = 8;
  cr_drum_spd1 = 16;
  cr_drum_spd2 = 32;
  cr_drum_spd3 = 64;
  cr_drum_spd4 = 128;
{ define bits in status register }
  sr_spd_srvo_lck_lost = 1;
  sr_carr_index        = 32;
  sr_step_index        = 64;
  sr_reset_not         = 128;

type
  dir_type = (forw,reverse);
  pos_neg_type  = (positive,negative);
  Parray = array [1..9] of word;
  aray_type = array [0..max_line] of byte;
  file_name_type = string[file_name_len];
  num_beams_type = 1..8;
  poly_array = array [1..4] of integer;
  flash_type = (flash_it,draw_it,both);
  aperture_type = (circle,ellipse,oval,rectangle,round_rectangle,
                thermal);
  plot_line = array [0..max_line] of byte;

{ pointer type declarations }
  flash_ptr_type = ^flash;
  line_ptr_type  = ^line;
  dcode_ptr_type = ^d_code;
  band_ptr_type  = ^plot_line;
  block_ptr_type = ^block;

{ records }
  block = record                { holds a block of bytes containing }
    bit_block   : array [0..2] of byte;
  end;

  d_code = record               { holds data about an aperture }
    aperture    : aperture_type;
    x_size      : integer;
    y_size      : integer;
    x_radius    : integer;
    y_radius    : integer;
    hole_size   : integer;
    x_block_size        : integer;
    y_block_size : integer;
    x_center    : integer;
    y_center    : integer;
    y_offset    : integer;
    d_num       : integer;      { for debug purposes only }
    bit_block   : block_ptr_type;
  end;

  flash = record      { holds data for one aperture to be flashed at a loc }
    x : integer;
    y : integer;
    next_flash : flash_ptr_type;
    this_apt : dcode_ptr_type;
    this_hole : dcode_ptr_type;
  end;

  line = record       { holds data for one line to be drawn from loc to loc }
    width : integer;
    x1,x2,y1,y2 : integer;
    this_apt   : dcode_ptr_type;
    next_line  : line_ptr_type;
  end;

  ptrrec = record     { defines a pointer }
    ofs,seg : word;
  end;

  window = record
        xmin,xmax,ymin,ymax,scale,xorg,yorg : integer;
  end;

  { the following is part of the typecast trickery for variable array sizes }
  bits_type = array [0..max_ap_area] of byte;

var
  params     : parray;
  mode,flag:integer;
  result                : integer;
  ar_ptr,array_ptr,dma_buf      : ^aray_type;
  i,j,k,line_no,count,lines,iy    : integer;
  dma_seg,dma_offset,dma_actual : word;
  file_name     : file_name_type;
  ofset,segmnt : word;
  infile,picture : text;
  start,ending,spacing : integer;
  done,bflag : boolean;
  which_band,last_band : integer;
  flash_1    : flash_ptr_type;
  draw_1     : line_ptr_type;
  curr_window : window;      { holds corner coords of current marking band }
  achar       : char;

{ global variables for PDMA card driver }

{ global variables for plotting bands by interrupt }
  plotting_band : integer;      { remembers which band we are plotting }
  plotting_line : integer;      { remebers which line we are on }
  band_complete : boolean ;     { semaphore for check_band }
  int_bdma_save : procedure;    { saves previous interrupt service routine addr }
  intercom_semaphore : boolean; { tells whether there is a message pending }
  intercom_text : string[80];   { the message buffer }
  This_line : band_ptr_type;    { holds ptr to plot line currently being DMA'ed to plotter }

{ global variables for program setup }
  pos_neg : pos_neg_type;
  mirror : boolean;
  x_delta,x_refl : integer;
  lines_band : integer;
  line_length : integer;
  main_window : window;
  num_beams : num_beams_type;
  x_offset, y_offset : integer;
  band_width : integer;       { number of lines in a rasterizing band }
  border     : integer;
  band_yoffset : integer;
  Apertures_loaded : boolean;
  Gerber_loaded : boolean;

{ global vars for statistics }
  flash_count : integer;
  draw_count_point,draw_count_vert,draw_count_horiz,draw_count_diag : integer;

{ instantiate pointers }
  flash_head,flash_tail : flash_ptr_type;
  line_head,line_tail   : line_ptr_type;
  a_flash       : flash_ptr_type;
  a_line        : line_ptr_type;


{ build arrays }

  dcode_array : array [0..max_aperture] of dcode_ptr_type;

{ these arrays contain pointer references to (possibly discontinuous) plot line buffers }
  band1_array, band2_array : array [0..max_band_length] of band_ptr_type;
  band1_actual,band2_actual : array[0..max_band_length] of integer;
        { holds actual parameter from tpdma allocate func so we can release mem }
  band1_handle,band2_handle : array[0..max_band_length] of THandle;
	  { holds globalhandles of band buffers }	
  line_mask : array [0..7] of byte;     { bit masks for inserting lines into
                                        band buffer or aperture arrays }
implementation
end.
